@@ -109,7 +109,7 @@ static void setup_private_mount(const char *snap_name)
 }
 
 // TODO: fold this into bootstrap
-static void setup_private_pts()
+static void setup_private_pts(void)
 {
 	// See https://www.kernel.org/doc/Documentation/filesystems/devpts.txt
 	//
@@ -165,7 +165,10 @@ static void sc_setup_mount_profiles(int snap_update_ns_fd,
 			"snap-update-ns", "--from-snap-confine", snap_name_copy,
 			NULL
 		};
-		char *envp[] = { NULL };
+		char *envp[3] = { NULL };
+		if (sc_is_debug_enabled()) {
+			envp[0] = "SNAPD_DEBUG=1";
+		}
 		debug("fexecv(%d (snap-update-ns), %s %s %s,)",
 		      snap_update_ns_fd, argv[0], argv[1], argv[2]);
 		fexecve(snap_update_ns_fd, argv, envp);
@@ -486,7 +489,7 @@ static void sc_bootstrap_mount_namespace(const struct sc_mount_config *config)
 static char * __attribute__ ((used))
     get_nextpath(char *path, size_t * offsetp, size_t fulllen)
 {
-	int offset = *offsetp;
+	size_t offset = *offsetp;
 
 	if (offset >= fulllen)
 		return NULL;
@@ -532,7 +535,7 @@ static bool __attribute__ ((used))
 	return false;
 }
 
-static int sc_open_snap_update_ns()
+static int sc_open_snap_update_ns(void)
 {
 	// +1 is for the case where the link is exactly PATH_MAX long but we also
 	// want to store the terminating '\0'. The readlink system call doesn't add
@@ -700,10 +703,15 @@ static bool is_mounted_with_shared_option(const char *dir)
 	return false;
 }
 
-void sc_ensure_shared_snap_mount()
+void sc_ensure_shared_snap_mount(void)
 {
 	if (!is_mounted_with_shared_option("/")
 	    && !is_mounted_with_shared_option(SNAP_MOUNT_DIR)) {
+		// TODO: We could be more aggressive and refuse to function but since
+		// we have no data on actual environments that happen to limp along in
+		// this configuration let's not do that yet.  This code should be
+		// removed once we have a measurement and feedback mechanism that lets
+		// us decide based on measurable data.
 		sc_do_mount(SNAP_MOUNT_DIR, SNAP_MOUNT_DIR, "none",
 			    MS_BIND | MS_REC, 0);
 		sc_do_mount("none", SNAP_MOUNT_DIR, NULL, MS_SHARED | MS_REC,
