@@ -25,20 +25,29 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/builtin"
 	"github.com/snapcore/snapd/interfaces/systemd"
+	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/testutil"
 )
 
 type GpioInterfaceSuite struct {
-	iface                   interfaces.Interface
-	gadgetGpioSlot          *interfaces.Slot
-	gadgetMissingNumberSlot *interfaces.Slot
-	gadgetBadNumberSlot     *interfaces.Slot
-	gadgetBadInterfaceSlot  *interfaces.Slot
-	gadgetPlug              *interfaces.Plug
-	gadgetBadInterfacePlug  *interfaces.Plug
-	osGpioSlot              *interfaces.Slot
-	appGpioSlot             *interfaces.Slot
+	iface                       interfaces.Interface
+	gadgetGpioSlotInfo          *snap.SlotInfo
+	gadgetGpioSlot              *interfaces.ConnectedSlot
+	gadgetMissingNumberSlotInfo *snap.SlotInfo
+	gadgetMissingNumberSlot     *interfaces.ConnectedSlot
+	gadgetBadNumberSlotInfo     *snap.SlotInfo
+	gadgetBadNumberSlot         *interfaces.ConnectedSlot
+	gadgetBadInterfaceSlotInfo  *snap.SlotInfo
+	gadgetBadInterfaceSlot      *interfaces.ConnectedSlot
+	gadgetPlugInfo              *snap.PlugInfo
+	gadgetPlug                  *interfaces.ConnectedPlug
+	gadgetBadInterfacePlugInfo  *snap.PlugInfo
+	gadgetBadInterfacePlug      *interfaces.ConnectedPlug
+	osGpioSlotInfo              *snap.SlotInfo
+	osGpioSlot                  *interfaces.ConnectedSlot
+	appGpioSlotInfo             *snap.SlotInfo
+	appGpioSlot                 *interfaces.ConnectedSlot
 }
 
 var _ = Suite(&GpioInterfaceSuite{
@@ -63,12 +72,18 @@ plugs:
     plug: gpio
     bad-interface-plug: other-interface
 `, nil)
-	s.gadgetGpioSlot = &interfaces.Slot{SlotInfo: gadgetInfo.Slots["my-pin"]}
-	s.gadgetMissingNumberSlot = &interfaces.Slot{SlotInfo: gadgetInfo.Slots["missing-number"]}
-	s.gadgetBadNumberSlot = &interfaces.Slot{SlotInfo: gadgetInfo.Slots["bad-number"]}
-	s.gadgetBadInterfaceSlot = &interfaces.Slot{SlotInfo: gadgetInfo.Slots["bad-interface-slot"]}
-	s.gadgetPlug = &interfaces.Plug{PlugInfo: gadgetInfo.Plugs["plug"]}
-	s.gadgetBadInterfacePlug = &interfaces.Plug{PlugInfo: gadgetInfo.Plugs["bad-interface-plug"]}
+	s.gadgetGpioSlotInfo = gadgetInfo.Slots["my-pin"]
+	s.gadgetGpioSlot = interfaces.NewConnectedSlot(s.gadgetGpioSlotInfo, nil)
+	s.gadgetMissingNumberSlotInfo = gadgetInfo.Slots["missing-number"]
+	s.gadgetMissingNumberSlot = interfaces.NewConnectedSlot(s.gadgetMissingNumberSlotInfo, nil)
+	s.gadgetBadNumberSlotInfo = gadgetInfo.Slots["bad-number"]
+	s.gadgetBadNumberSlot = interfaces.NewConnectedSlot(s.gadgetBadNumberSlotInfo, nil)
+	s.gadgetBadInterfaceSlotInfo = gadgetInfo.Slots["bad-interface-slot"]
+	s.gadgetBadInterfaceSlot = interfaces.NewConnectedSlot(s.gadgetBadInterfaceSlotInfo, nil)
+	s.gadgetPlugInfo = gadgetInfo.Plugs["plug"]
+	s.gadgetPlug = interfaces.NewConnectedPlug(s.gadgetPlugInfo, nil)
+	s.gadgetBadInterfacePlugInfo = gadgetInfo.Plugs["bad-interface-plug"]
+	s.gadgetBadInterfacePlug = interfaces.NewConnectedPlug(s.gadgetBadInterfacePlugInfo, nil)
 
 	osInfo := snaptest.MockInfo(c, `
 name: my-core
@@ -79,7 +94,8 @@ slots:
         number: 777
         direction: out
 `, nil)
-	s.osGpioSlot = &interfaces.Slot{SlotInfo: osInfo.Slots["my-pin"]}
+	s.osGpioSlotInfo = osInfo.Slots["my-pin"]
+	s.osGpioSlot = interfaces.NewConnectedSlot(s.osGpioSlotInfo, nil)
 
 	appInfo := snaptest.MockInfo(c, `
 name: my-app
@@ -89,7 +105,8 @@ slots:
         number: 154
         direction: out
 `, nil)
-	s.appGpioSlot = &interfaces.Slot{SlotInfo: appInfo.Slots["my-pin"]}
+	s.appGpioSlotInfo = appInfo.Slots["my-pin"]
+	s.appGpioSlot = interfaces.NewConnectedSlot(s.appGpioSlotInfo, nil)
 }
 
 func (s *GpioInterfaceSuite) TestName(c *C) {
@@ -98,35 +115,35 @@ func (s *GpioInterfaceSuite) TestName(c *C) {
 
 func (s *GpioInterfaceSuite) TestSanitizeSlotGadgetSnap(c *C) {
 	// gpio slot on gadget accepeted
-	c.Assert(s.gadgetGpioSlot.Sanitize(s.iface), IsNil)
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.gadgetGpioSlotInfo), IsNil)
 
 	// slots without number attribute are rejected
-	c.Assert(s.gadgetMissingNumberSlot.Sanitize(s.iface), ErrorMatches,
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.gadgetMissingNumberSlotInfo), ErrorMatches,
 		"gpio slot must have a number attribute")
 
 	// slots with number attribute that isnt a number
-	c.Assert(s.gadgetBadNumberSlot.Sanitize(s.iface), ErrorMatches,
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.gadgetBadNumberSlotInfo), ErrorMatches,
 		"gpio slot number attribute must be an int")
 }
 
 func (s *GpioInterfaceSuite) TestSanitizeSlotOsSnap(c *C) {
 	// gpio slot on OS accepeted
-	c.Assert(s.osGpioSlot.Sanitize(s.iface), IsNil)
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.osGpioSlotInfo), IsNil)
 }
 
 func (s *GpioInterfaceSuite) TestSanitizeSlotAppSnap(c *C) {
 	// gpio slot not accepted on app snap
-	c.Assert(s.appGpioSlot.Sanitize(s.iface), ErrorMatches,
+	c.Assert(interfaces.BeforePrepareSlot(s.iface, s.appGpioSlotInfo), ErrorMatches,
 		"gpio slots are reserved for the core and gadget snaps")
 }
 
 func (s *GpioInterfaceSuite) TestSanitizePlug(c *C) {
-	c.Assert(s.gadgetPlug.Sanitize(s.iface), IsNil)
+	c.Assert(interfaces.BeforePreparePlug(s.iface, s.gadgetPlugInfo), IsNil)
 }
 
 func (s *GpioInterfaceSuite) TestSystemdConnectedSlot(c *C) {
 	spec := &systemd.Specification{}
-	err := spec.AddConnectedSlot(s.iface, s.gadgetPlug, nil, s.gadgetGpioSlot, nil)
+	err := spec.AddConnectedSlot(s.iface, s.gadgetPlug, s.gadgetGpioSlot)
 	c.Assert(err, IsNil)
 	c.Assert(spec.Services(), DeepEquals, map[string]*systemd.Service{
 		"snap.my-device.interface.gpio-100.service": {
