@@ -56,11 +56,20 @@ dbus (send)
     peer=(label=unconfined),
 
 # freedesktop.org ScreenSaver
+# compatibility rule
 dbus (send)
     bus=session
-    path=/{,org/freedesktop/,org.gnome/}Screensaver
+    path=/Screensaver
     interface=org.freedesktop.ScreenSaver
-    member=org.freedesktop.ScreenSaver.{Inhibit,UnInhibit,SimulateUserActivity}
+    member={Inhibit,UnInhibit,SimulateUserActivity}
+    peer=(label=unconfined),
+
+# API rule
+dbus (send)
+    bus=session
+    path=/{,org/freedesktop/,org.gnome/}ScreenSaver
+    interface=org.freedesktop.ScreenSaver
+    member={Inhibit,UnInhibit,SimulateUserActivity}
     peer=(label=unconfined),
 
 # gnome, kde and cinnamon screensaver
