@@ -448,7 +448,7 @@ plugs:
 // describe installation rules for slots succinctly for cross-checking,
 // if an interface is not mentioned here a slot of its type can only
 // be installed by a core snap (and this was taken care by
-// SanitizeSlot),
+// BeforePrepareSlot),
 // otherwise the entry for the interface is the list of snap types it
 // can be installed by (using the declaration naming);
 // ATM a nil entry means even stricter rules that would need be tested
@@ -500,6 +500,7 @@ var (
 		"unity8-calendar":           {"app"},
 		"unity8-contacts":           {"app"},
 		"upower-observe":            {"app", "core"},
+		"wayland":                   {"app", "core"},
 		// snowflakes
 		"classic-support": nil,
 		"docker":          nil,
@@ -525,7 +526,7 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 		types, ok := slotInstallation[iface.Name()]
 		compareWithSanitize := false
 		if !ok { // common ones, only core can install them,
-			// their plain SanitizeSlot checked for that
+			// their plain BeforePrepareSlot checked for that
 			types = []string{"core"}
 			compareWithSanitize = true
 		}
@@ -545,8 +546,7 @@ func (s *baseDeclSuite) TestSlotInstallation(c *C) {
 				c.Check(err, NotNil, comm)
 			}
 			if compareWithSanitize {
-				slot := &interfaces.Slot{SlotInfo: slotInfo}
-				sanitizeErr := slot.Sanitize(iface)
+				sanitizeErr := interfaces.BeforePrepareSlot(iface, slotInfo)
 				if err == nil {
 					c.Check(sanitizeErr, IsNil, comm)
 				} else {
@@ -707,6 +707,7 @@ func (s *baseDeclSuite) TestSanity(c *C) {
 		"lxd-support":           true,
 		"snapd-control":         true,
 		"unity8":                true,
+		"wayland":               true,
 	}
 
 	for _, iface := range all {
@@ -791,3 +792,17 @@ series: 16
 revision: 0
 `)
 }
+
+func (s *baseDeclSuite) TestBrowserSupportAllowSandbox(c *C) {
+	const plugYaml = `name: plug-snap
+plugs:
+  browser-support:
+   allow-sandbox: true
+`
+	cand := s.connectCand(c, "browser-support", "", plugYaml)
+	err := cand.Check()
+	c.Check(err, NotNil)
+
+	err = cand.CheckAutoConnect()
+	c.Check(err, NotNil)
+}
