@@ -25,6 +25,7 @@ import (
 	"fmt"
 	"io/ioutil"
 	"path/filepath"
+	"sort"
 	"testing"
 	"time"
 
@@ -41,7 +42,6 @@ import (
 	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
-	"github.com/snapcore/snapd/overlord/storestate"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/snap/snaptest"
 	"github.com/snapcore/snapd/store/storetest"
@@ -108,7 +108,7 @@ func (s *assertMgrSuite) SetUpTest(c *C) {
 	s.o.AddManager(s.mgr)
 
 	s.state.Lock()
-	storestate.ReplaceStore(s.state, &fakeStore{
+	snapstate.ReplaceStore(s.state, &fakeStore{
 		state: s.state,
 		db:    s.storeSigning,
 	})
@@ -127,6 +127,12 @@ func (s *assertMgrSuite) TestDB(c *C) {
 	c.Check(db, FitsTypeOf, (*asserts.Database)(nil))
 }
 
+func (s *assertMgrSuite) TestKnownTaskKinds(c *C) {
+	kinds := s.mgr.KnownTaskKinds()
+	sort.Strings(kinds)
+	c.Assert(kinds, DeepEquals, []string{"validate-snap"})
+}
+
 func (s *assertMgrSuite) TestAdd(c *C) {
 	s.state.Lock()
 	defer s.state.Unlock()
@@ -1131,3 +1137,29 @@ func (s *assertMgrSuite) TestPublisher(c *C) {
 	c.Check(acct.AccountID(), Equals, s.dev1Acct.AccountID())
 	c.Check(acct.Username(), Equals, "developer1")
 }
+
+func (s *assertMgrSuite) TestStore(c *C) {
+	s.state.Lock()
+	defer s.state.Unlock()
+
+	err := assertstate.Add(s.state, s.storeSigning.StoreAccountKey(""))
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, s.dev1Acct)
+	c.Assert(err, IsNil)
+	storeHeaders := map[string]interface{}{
+		"store":       "foo",
+		"operator-id": s.dev1Acct.AccountID(),
+		"timestamp":   time.Now().Format(time.RFC3339),
+	}
+	fooStore, err := s.storeSigning.Sign(asserts.StoreType, storeHeaders, nil, "")
+	c.Assert(err, IsNil)
+	err = assertstate.Add(s.state, fooStore)
+	c.Assert(err, IsNil)
+
+	_, err = assertstate.Store(s.state, "bar")
+	c.Check(asserts.IsNotFound(err), Equals, true)
+
+	store, err := assertstate.Store(s.state, "foo")
+	c.Assert(err, IsNil)
+	c.Check(store.Store(), Equals, "foo")
+}
