@@ -40,7 +40,9 @@ import (
 	"github.com/snapcore/snapd/overlord"
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/auth"
+	"github.com/snapcore/snapd/overlord/configstate"
 	"github.com/snapcore/snapd/overlord/configstate/config"
+	"github.com/snapcore/snapd/overlord/configstate/configcore"
 	"github.com/snapcore/snapd/overlord/devicestate"
 	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/ifacestate"
@@ -743,6 +745,13 @@ snaps:
 	rhk := hookstate.MockRunHook(hookInvoke)
 	defer rhk()
 
+	// ensure we have something that captures the core config
+	restore := configstate.MockConfigcoreRun(func(configcore.Conf) error {
+		configured = append(configured, "configcore")
+		return nil
+	})
+	defer restore()
+
 	// avoid device reg
 	chg1 := st.NewChange("become-operational", "init device")
 	chg1.SetStatus(state.DoingStatus)
@@ -800,7 +809,7 @@ snaps:
 	c.Assert(err, IsNil)
 	c.Check(val, Equals, "foo.")
 
-	c.Check(configured, DeepEquals, []string{"core", "pc-kernel", "pc", "foo"})
+	c.Check(configured, DeepEquals, []string{"configcore", "pc-kernel", "pc", "foo"})
 
 	// and ensure state is now considered seeded
 	var seeded bool
