@@ -22,7 +22,6 @@ package snapstate
 import (
 	"fmt"
 	"os"
-	"path/filepath"
 	"strconv"
 	"strings"
 	"time"
@@ -30,17 +29,14 @@ import (
 	"gopkg.in/tomb.v2"
 
 	"github.com/snapcore/snapd/boot"
-	"github.com/snapcore/snapd/dirs"
 	"github.com/snapcore/snapd/logger"
-	"github.com/snapcore/snapd/osutil"
+	"github.com/snapcore/snapd/overlord/auth"
 	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/snapstate/backend"
 	"github.com/snapcore/snapd/overlord/state"
-	"github.com/snapcore/snapd/overlord/storestate"
 	"github.com/snapcore/snapd/release"
 	"github.com/snapcore/snapd/snap"
 	"github.com/snapcore/snapd/store"
-	"github.com/snapcore/snapd/strutil"
 )
 
 // TaskSnapSetup returns the SnapSetup with task params hold by or referred to by the the task.
@@ -325,7 +321,7 @@ func (m *SnapManager) doDownloadSnap(t *state.Task, tomb *tomb.Tomb) error {
 	}
 
 	st.Lock()
-	theStore := storestate.Store(st)
+	theStore := Store(st)
 	user, err := userFromUserID(st, snapsup.UserID)
 	st.Unlock()
 	if err != nil {
@@ -482,10 +478,6 @@ func (m *SnapManager) undoUnlinkCurrentSnap(t *state.Task, _ *tomb.Tomb) error {
 		return err
 	}
 
-	if err := m.createSnapCookie(st, snapsup.Name()); err != nil {
-		return fmt.Errorf("cannot create snap cookie: %v", err)
-	}
-
 	snapst.Active = true
 	err = m.backend.LinkSnap(oldInfo)
 	if err != nil {
@@ -571,49 +563,6 @@ func (m *SnapManager) cleanupCopySnapData(t *state.Task, _ *tomb.Tomb) error {
 	return nil
 }
 
-func (m *SnapManager) createSnapCookie(st *state.State, snapName string) error {
-	cookieID := strutil.MakeRandomString(44)
-	path := filepath.Join(dirs.SnapCookieDir, fmt.Sprintf("snap.%s", snapName))
-	err := osutil.AtomicWriteFile(path, []byte(cookieID), 0600, 0)
-	if err != nil {
-		return err
-	}
-
-	var contexts map[string]string
-	err = st.Get("snap-cookies", &contexts)
-	if err != nil {
-		if err != state.ErrNoState {
-			return fmt.Errorf("cannot get snap cookies: %v", err)
-		}
-		contexts = make(map[string]string)
-	}
-	contexts[cookieID] = snapName
-	st.Set("snap-cookies", &contexts)
-	return nil
-}
-
-func (m *SnapManager) removeSnapCookie(st *state.State, snapName string) error {
-	var contexts map[string]string
-	err := st.Get("snap-cookies", &contexts)
-	if err != nil {
-		if err != state.ErrNoState {
-			return fmt.Errorf("cannot get snap cookies: %v", err)
-		}
-		contexts = make(map[string]string)
-	}
-
-	for cookieID, snap := range contexts {
-		if snapName == snap {
-			delete(contexts, cookieID)
-			st.Set("snap-cookies", contexts)
-			// error on removing the context file is not fatal
-			_ = os.Remove(filepath.Join(dirs.SnapCookieDir, fmt.Sprintf("snap.%s", snapName)))
-			break
-		}
-	}
-	return nil
-}
-
 func (m *SnapManager) doLinkSnap(t *state.Task, _ *tomb.Tomb) error {
 	st := t.State()
 	st.Lock()
@@ -657,6 +606,23 @@ func (m *SnapManager) doLinkSnap(t *state.Task, _ *tomb.Tomb) error {
 	if snapsup.Required { // set only on install and left alone on refresh
 		snapst.Required = true
 	}
+	// only set userID if unset or logged out in snapst and if we
+	// actually have an associated user
+	if snapsup.UserID > 0 {
+		var user *auth.UserState
+		if snapst.UserID != 0 {
+			user, err = auth.User(st, snapst.UserID)
+			if err != nil && err != auth.ErrInvalidUser {
+				return err
+			}
+		}
+		if user == nil {
+			// if the original user installing the snap is
+			// no longer available transfer to user who
+			// triggered this change
+			snapst.UserID = snapsup.UserID
+		}
+	}
 
 	newInfo, err := readInfo(snapsup.Name(), cand)
 	if err != nil {
@@ -779,7 +745,7 @@ func (m *SnapManager) undoLinkSnap(t *state.Task, _ *tomb.Tomb) error {
 
 	if len(snapst.Sequence) == 1 {
 		if err := m.removeSnapCookie(st, snapsup.Name()); err != nil {
-			return fmt.Errorf("cannot remove snap context: %v", err)
+			return fmt.Errorf("cannot remove snap cookie: %v", err)
 		}
 	}
 
@@ -863,6 +829,38 @@ func (m *SnapManager) doSwitchSnapChannel(t *state.Task, _ *tomb.Tomb) error {
 	return nil
 }
 
+func (m *SnapManager) doSwitchSnap(t *state.Task, _ *tomb.Tomb) error {
+	st := t.State()
+	st.Lock()
+	defer st.Unlock()
+
+	snapsup, snapst, err := snapSetupAndState(t)
+	if err != nil {
+		return err
+	}
+	snapst.Channel = snapsup.Channel
+
+	Set(st, snapsup.Name(), snapst)
+	return nil
+}
+
+func (m *SnapManager) doToggleSnapFlags(t *state.Task, _ *tomb.Tomb) error {
+	st := t.State()
+	st.Lock()
+	defer st.Unlock()
+
+	snapsup, snapst, err := snapSetupAndState(t)
+	if err != nil {
+		return err
+	}
+
+	// for now we support toggling only ignore-validation
+	snapst.IgnoreValidation = snapsup.IgnoreValidation
+
+	Set(st, snapsup.Name(), snapst)
+	return nil
+}
+
 func (m *SnapManager) startSnapServices(t *state.Task, _ *tomb.Tomb) error {
 	st := t.State()
 	st.Lock()
@@ -1027,7 +1025,7 @@ func (m *SnapManager) doDiscardSnap(t *state.Task, _ *tomb.Tomb) error {
 			return &state.Retry{After: 3 * time.Minute}
 		}
 		if err := m.removeSnapCookie(st, snapsup.Name()); err != nil {
-			return fmt.Errorf("cannot remove snap context: %v", err)
+			return fmt.Errorf("cannot remove snap cookie: %v", err)
 		}
 	}
 	if err = config.DiscardRevisionConfig(st, snapsup.Name(), snapsup.Revision()); err != nil {
