@@ -8,6 +8,8 @@ set -eux
 . "$TESTSLIB/snaps.sh"
 # shellcheck source=tests/lib/pkgdb.sh
 . "$TESTSLIB/pkgdb.sh"
+# shellcheck source=tests/lib/boot.sh
+. "$TESTSLIB/boot.sh"
 
 disable_kernel_rate_limiting() {
     # kernel rate limiting hinders debugging security policy so turn it off
@@ -18,6 +20,54 @@ disable_kernel_rate_limiting() {
     #sysctl -w kernel.printk_ratelimit=0
 }
 
+disable_refreshes() {
+    echo "Ensure jq is available"
+    if ! which jq; then
+        snap install --devmode jq
+    fi
+
+    echo "Modify state to make it look like the last refresh just happened"
+    systemctl stop snapd.socket snapd.service
+    jq ".data[\"last-refresh\"] = \"$(date +%Y-%m-%dT%H:%M:%S%:z)\"" /var/lib/snapd/state.json > /var/lib/snapd/state.json.new
+    mv /var/lib/snapd/state.json.new /var/lib/snapd/state.json
+    systemctl start snapd.socket snapd.service
+
+    echo "Minimize risk of hitting refresh schedule"
+    snap set core refresh.schedule=00:00-23:59
+    snap refresh --time|MATCH "last: 2[0-9]{3}"
+
+    echo "Ensure jq is gone"
+    snap remove jq
+}
+
+setup_systemd_snapd_overrides() {
+    START_LIMIT_INTERVAL="StartLimitInterval=0"
+    if [[ "$SPREAD_SYSTEM" = opensuse-42.2-* ]]; then
+        # StartLimitInterval is not supported by the systemd version
+        # openSUSE 42.2 ships.
+        START_LIMIT_INTERVAL=""
+    fi
+
+    mkdir -p /etc/systemd/system/snapd.service.d
+    cat <<EOF > /etc/systemd/system/snapd.service.d/local.conf
+[Unit]
+$START_LIMIT_INTERVAL
+[Service]
+Environment=SNAPD_DEBUG_HTTP=7 SNAPD_DEBUG=1 SNAPPY_TESTING=1 SNAPD_CONFIGURE_HOOK_TIMEOUT=30s SNAPPY_USE_STAGING_STORE=$SNAPPY_USE_STAGING_STORE
+ExecStartPre=/bin/touch /dev/iio:device0
+EOF
+    mkdir -p /etc/systemd/system/snapd.socket.d
+    cat <<EOF > /etc/systemd/system/snapd.socket.d/local.conf
+[Unit]
+$START_LIMIT_INTERVAL
+EOF
+
+    # We change the service configuration so reload and restart
+    # the snapd socket unit to get them applied
+    systemctl daemon-reload
+    systemctl restart snapd.socket
+}
+
 update_core_snap_for_classic_reexec() {
     # it is possible to disable this to test that snapd (the deb) works
     # fine with whatever is in the core snap
@@ -44,6 +94,9 @@ update_core_snap_for_classic_reexec() {
     cp -a "$LIBEXECDIR"/snapd/* squashfs-root/usr/lib/snapd/
     # also the binaries themselves
     cp -a /usr/bin/{snap,snapctl} squashfs-root/usr/bin/
+    # clean and install the udev helper
+    rm -f squashfs-root//lib/udev/snappy-app-dev
+    cp -a /lib/udev/snappy-app-dev squashfs-root/lib/udev/
 
     case "$SPREAD_SYSTEM" in
         ubuntu-*|debian-*)
@@ -131,30 +184,7 @@ prepare_classic() {
         exit 1
     fi
 
-    START_LIMIT_INTERVAL="StartLimitInterval=0"
-    if [[ "$SPREAD_SYSTEM" = opensuse-42.2-* ]]; then
-        # StartLimitInterval is not supported by the systemd version
-        # openSUSE 42.2 ships.
-        START_LIMIT_INTERVAL=""
-    fi
-
-    mkdir -p /etc/systemd/system/snapd.service.d
-    cat <<EOF > /etc/systemd/system/snapd.service.d/local.conf
-[Unit]
-$START_LIMIT_INTERVAL
-[Service]
-Environment=SNAPD_DEBUG_HTTP=7 SNAPD_DEBUG=1 SNAPPY_TESTING=1 SNAPD_CONFIGURE_HOOK_TIMEOUT=30s
-EOF
-    mkdir -p /etc/systemd/system/snapd.socket.d
-    cat <<EOF > /etc/systemd/system/snapd.socket.d/local.conf
-[Unit]
-$START_LIMIT_INTERVAL
-EOF
-
-    # We change the service configuration so reload and restart
-    # the snapd socket unit to get them applied
-    systemctl daemon-reload
-    systemctl restart snapd.socket
+    setup_systemd_snapd_overrides
 
     if [ "$REMOTE_STORE" = staging ]; then
         # shellcheck source=tests/lib/store.sh
@@ -173,18 +203,15 @@ EOF
         # used for resuming downloads.
         (
             set -x
-            cd /tmp
+            cd "$TESTSLIB/cache/"
+            # Download each of the snaps we want to pre-cache. Note that `snap download`
+            # a quick no-op if the file is complete.
             for snap_name in ${PRE_CACHE_SNAPS:-}; do
                 snap download "$snap_name"
             done
-            for snap_file in *.snap; do
-                mv "$snap_file" "$snap_file.partial"
-                # There is a bug in snapd where partial file must be a proper
-                # prefix of the full file or we make a wrong request to the
-                # store.
-                truncate --size=-1 "$snap_file.partial"
-                mv "$snap_file.partial" /var/lib/snapd/snaps/
-            done
+            # Copy all of the snaps back to the spool directory. From there we
+            # will reuse them during subsequent `snap install` operations.
+            cp *.snap /var/lib/snapd/snaps/
             set +x
         )
 
@@ -199,23 +226,13 @@ EOF
         update_core_snap_for_classic_reexec
         systemctl start snapd.{service,socket}
 
-        # ensure no auto-refresh happens during the tests
-        if [ -e /snap/core/current/meta/hooks/configure ]; then
-            snap set core refresh.schedule="$(date +%a --date=2days)@12:00-14:00"
-            snap set core refresh.disabled=true
-        fi
+        disable_refreshes
 
-        GRUB_EDITENV=grub-editenv
-        case "$SPREAD_SYSTEM" in
-            fedora-*|opensuse-*)
-                GRUB_EDITENV=grub2-editenv
-                ;;
-        esac
-
-        echo "Ensure that the grub-editenv list output does not contain any of the snap_* variables on classic"
-        output=$($GRUB_EDITENV list)
+        echo "Ensure that the bootloader environment output does not contain any of the snap_* variables on classic"
+        # shellcheck disable=SC2119
+        output=$(bootenv)
         if echo "$output" | MATCH snap_ ; then
-            echo "Expected grub environment without snap_*, got:"
+            echo "Expected bootloader environment without snap_*, got:"
             echo "$output"
             exit 1
         fi
@@ -229,7 +246,7 @@ EOF
         done
         snapd_env="/etc/environment /etc/systemd/system/snapd.service.d /etc/systemd/system/snapd.socket.d"
         # shellcheck disable=SC2086
-        tar czf "$SPREAD_PATH"/snapd-state.tar.gz /var/lib/snapd "$SNAP_MOUNT_DIR" /etc/systemd/system/"$escaped_snap_mount_dir"-*core*.mount $snapd_env
+        tar cf "$SPREAD_PATH"/snapd-state.tar.gz /var/lib/snapd "$SNAP_MOUNT_DIR" /etc/systemd/system/"$escaped_snap_mount_dir"-*core*.mount /etc/systemd/system/multi-user.target.wants/"$escaped_snap_mount_dir"-*core*.mount $snapd_env
         systemctl daemon-reload # Workaround for http://paste.ubuntu.com/17735820/
         core="$(readlink -f "$SNAP_MOUNT_DIR/core/current")"
         # on 14.04 it is possible that the core snap is still mounted at this point, unmount
@@ -243,30 +260,6 @@ EOF
         systemctl start snapd.socket
     fi
 
-    if [[ "$SPREAD_SYSTEM" == debian-* || "$SPREAD_SYSTEM" == ubuntu-* ]]; then
-        if [[ "$SPREAD_SYSTEM" == ubuntu-* ]]; then
-            pollinate
-        fi
-
-        # Improve entropy for the whole system quite a lot to get fast
-        # key generation during our test cycles
-        echo "HRNGDEVICE=/dev/urandom" > /etc/default/rng-tools
-        /etc/init.d/rng-tools restart
-
-        if systemctl list-units | MATCH "rng-tools.service"; then
-            mkdir -p /etc/systemd/system/rng-tools.service.d/
-            cat <<EOF > /etc/systemd/system/rng-tools.service.d/local.conf
-[Service]
-Restart=always
-RestartSec=2
-RemainAfterExit=no
-PIDFile=/var/run/rngd.pid
-EOF
-        systemctl daemon-reload
-        systemctl restart rng-tools.service
-        fi
-    fi
-
     disable_kernel_rate_limiting
 }
 
@@ -294,30 +287,6 @@ setup_reflash_magic() {
         # FIXME: netplan workaround
         mkdir -p "$UNPACKD/etc/netplan"
 
-        # set root pw by concating root line from host and rest from core
-        want_pw="$(grep ^root /etc/shadow)"
-        echo "$want_pw" > /tmp/new-shadow
-        tail -n +2 /etc/shadow >> /tmp/new-shadow
-        cp -v /tmp/new-shadow "$UNPACKD/etc/shadow"
-        cp -v /etc/passwd "$UNPACKD/etc/passwd"
-
-        # ensure spread -reuse works in the core image as well
-        if [ -e /.spread.yaml ]; then
-            cp -av /.spread.yaml "$UNPACKD"
-        fi
-
-        # we need the test user in the image
-        # see the comment in spread.yaml about 12345
-        sed -i 's/^test.*$//' "$UNPACKD"/etc/{shadow,passwd}
-        chroot "$UNPACKD" addgroup --quiet --gid 12345 test
-        chroot "$UNPACKD" adduser --quiet --no-create-home --uid 12345 --gid 12345 --disabled-password --gecos '' test
-        echo 'test ALL=(ALL) NOPASSWD:ALL' >> "$UNPACKD/etc/sudoers.d/99-test-user"
-
-        echo 'ubuntu ALL=(ALL) NOPASSWD:ALL' >> "$UNPACKD/etc/sudoers.d/99-ubuntu-user"
-
-        # modify sshd so that we can connect as root
-        sed -i 's/\(PermitRootLogin\|PasswordAuthentication\)\>.*/\1 yes/' "$UNPACKD/etc/ssh/sshd_config"
-
         # FIXME: install would be better but we don't have dpkg on
         #        the image
         # unpack our freshly build snapd into the new core snap
@@ -380,8 +349,64 @@ EOF
         mkdir -p /mnt/user-data/
         cp -ar /home/gopath /mnt/user-data/
 
-        # create test user home dir
+        # create test user and ubuntu user inside the writable partition
+        # so that we can use a stock core in tests
         mkdir -p /mnt/user-data/test
+
+        # create test user, see the comment in spread.yaml about 12345
+        mkdir -p /mnt/system-data/etc/sudoers.d/
+        echo 'test ALL=(ALL) NOPASSWD:ALL' >> /mnt/system-data/etc/sudoers.d/99-test-user
+        echo 'ubuntu ALL=(ALL) NOPASSWD:ALL' >> /mnt/system-data/etc/sudoers.d/99-ubuntu-user
+        # modify sshd so that we can connect as root
+        mkdir -p /mnt/system-data/etc/ssh
+        cp -a "$UNPACKD"/etc/ssh/* /mnt/system-data/etc/ssh/
+        sed -i 's/\(PermitRootLogin\|PasswordAuthentication\)\>.*/\1 yes/' /mnt/system-data/etc/ssh/sshd_config
+
+        # build the user database - this is complicated because:
+        # - spread on linode wants to login as "root"
+        # - "root" login on the stock core snap is disabled
+        # - we need to add our ubuntu and test users too
+        # - uids between classic/core differ
+        # - passwd,shadow on core are read-only
+        # - we cannot add root to extrausers as system passwd is searched first
+        # So we do:
+        # - take core passwd without "root" as extrausers
+        # - append root,ubuntu,test to extrausers
+        # - bind mount extrausers to /etc via custom systemd job
+        mkdir -p /mnt/system-data/var/lib/extrausers/
+        touch /mnt/system-data/var/lib/extrausers/sub{uid,gid}
+        mkdir -p /mnt/system-data/etc/systemd/system/multi-user.target.wants
+        for f in group gshadow passwd shadow; do
+            # the passwd from core without root
+            tail -n +2 "$UNPACKD/etc/$f" > /mnt/system-data/var/lib/extrausers/$f
+            # append this systems root user so that linode can connect
+            head -n1 /etc/$f >> /mnt/system-data/var/lib/extrausers/$f
+            # append ubuntu, test user for the testing
+            tail -n2 /etc/$f >> /mnt/system-data/var/lib/extrausers/$f
+
+            # now bind mount those passwd files on boot
+            cat <<EOF > /mnt/system-data/etc/systemd/system/etc-$f.mount
+[Unit]
+Description=Mount extrausers $f over system $f
+Before=ssh.service
+
+[Mount]
+What=/var/lib/extrausers/$f
+Where=/etc/$f
+Type=none
+Options=bind
+
+[Install]
+WantedBy=multi-user.target
+EOF
+            ln -s /etc/systemd/system/etc-$f.mount /mnt/system-data/etc/systemd/system/multi-user.target.wants/etc-$f.mount
+        done
+
+        # ensure spread -reuse works in the core image as well
+        if [ -e /.spread.yaml ]; then
+            cp -av /.spread.yaml /mnt/system-data
+        fi
+
         # using symbolic names requires test:test have the same ids
         # inside and outside which is a pain (see 12345 above), but
         # using the ids directly is the wrong kind of fragile
@@ -394,20 +419,6 @@ EOF
         (cd /tmp ; unsquashfs -v "$IMAGE_HOME"/core_*.snap etc/systemd/system)
         cp -avr /tmp/squashfs-root/etc/systemd/system /mnt/system-data/etc/systemd/
 
-        # FIXUP silly systemd
-        mkdir -p /mnt/system-data/etc/systemd/system/snapd.service.d
-        cat <<EOF > /mnt/system-data/etc/systemd/system/snapd.service.d/local.conf
-[Unit]
-StartLimitInterval=0
-[Service]
-Environment=SNAPD_DEBUG_HTTP=7 SNAPD_DEBUG=1 SNAPPY_TESTING=1 SNAPPY_USE_STAGING_STORE=$SNAPPY_USE_STAGING_STORE
-ExecStartPre=/bin/touch /dev/iio:device0
-EOF
-        mkdir -p /mnt/system-data/etc/systemd/system/snapd.socket.d
-        cat <<EOF > /mnt/system-data/etc/systemd/system/snapd.socket.d/local.conf
-[Unit]
-StartLimitInterval=0
-EOF
 
         umount /mnt
         kpartx -d  "$IMAGE_HOME/$IMAGE"
@@ -475,11 +486,8 @@ prepare_all_snap() {
         fi
     done
 
-    # ensure no auto-refresh happens during the tests
-    if [ -e /snap/core/current/meta/hooks/configure ]; then
-        snap set core refresh.schedule="$(date +%a --date=2days)@12:00-14:00"
-        snap set core refresh.disabled=true
-    fi
+    disable_refreshes
+    setup_systemd_snapd_overrides
 
     # Snapshot the fresh state (including boot/bootenv)
     if [ ! -f "$SPREAD_PATH/snapd-state.tar.gz" ]; then
@@ -497,7 +505,7 @@ prepare_all_snap() {
         fi
 
         systemctl stop snapd.service snapd.socket
-        tar czf "$SPREAD_PATH/snapd-state.tar.gz" /var/lib/snapd $BOOT /etc/systemd/system/snap-*core*.mount
+        tar cf "$SPREAD_PATH/snapd-state.tar.gz" /var/lib/snapd $BOOT /etc/systemd/system/snap-*core*.mount
         systemctl start snapd.socket
     fi
 
